package model.questionnaire;

import com.github.cliftonlabs.json_simple.JsonArray;
import com.github.cliftonlabs.json_simple.JsonObject;
import com.github.cliftonlabs.json_simple.Jsonable;
import model.personnality.Trait;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;

import static api.JsonKeys.*;

/**
 * Stores answers provided for a session of answers
 */
public class SessionAnswers implements Jsonable {

    private final HashMap<Question, Answer> fAnswers;
    private final HashMap<Trait, Float> fTraits;

    public SessionAnswers() {
        fTraits = new HashMap<>();
        fAnswers = new HashMap<>();
    }

    /**
     * Stores an answer to a question and updates the weight of the related trait.
     * If an answer has already been provided for the question, cancel the weight added by the outdated answer
     * @param iQuestion
     * @param iAnswer
     */
    public void setAnswer(Question iQuestion, Answer iAnswer) {
        // ---------------------------------
        // cancel the weight added by a potential previous answer
        // ---------------------------------
        if(fAnswers.containsKey(iQuestion)) {
            Answer vPreviousAnswer = fAnswers.get(iQuestion);
            Trait vPreviousTrait = vPreviousAnswer.getTrait();
            Float vOutdatedWeight = fTraits.get(vPreviousTrait);
            fTraits.put(vPreviousTrait, vOutdatedWeight - vPreviousAnswer.getWeight());
        }
        // ---------------------------------
        // Store the answer & update the related trait
        // ---------------------------------
        Trait vNewTrait = iAnswer.getTrait();
        Float vNewTraitWeight = iAnswer.getWeight();
        fAnswers.put(iQuestion, iAnswer);

        if(fTraits.containsKey(vNewTrait))
            vNewTraitWeight += fTraits.get(vNewTrait);

        fTraits.put(vNewTrait, vNewTraitWeight);
    }

    // ------------------------------------------------------------------
    // Jsonable Implementation
    // ------------------------------------------------------------------
    public JsonObject asJsonObject()
    {
        JsonObject vResult = new JsonObject();
        JsonArray vAnswers = new JsonArray();
        fAnswers.forEach( (question, answer) -> vAnswers.add(answer.asJsonObject().putChain(KEY_QUESTION, question.asJsonObject())));
        vResult.put(KEY_ANSWERS, vAnswers);
        JsonArray vTraits = new JsonArray();
        fTraits.forEach( (trait, weight) -> vTraits.add(trait.asJsonObject().putChain(KEY_WEIGHT, weight)));
        vResult.put(KEY_TRAITS, vTraits);
        return vResult;
    }


    @Override
    public String toJson() {
        final StringWriter writable = new StringWriter();
        try{
            this.toJson(writable);
        }catch(final IOException caught)
        {
            caught.printStackTrace();
        }
        return writable.toString();
    }

    @Override
    public void toJson(Writer writable) throws IOException {
        writable.write(new String(asJsonObject().toJson().getBytes(StandardCharsets.UTF_8), StandardCharsets.UTF_8));
    }

}
